////////////////////////////////////////////////////////////////////////////
//
// \brief Definition of shower object for LArSoft
//
// \author brebel@fnal.gov
//
////////////////////////////////////////////////////////////////////////////

#include "lardataobj/RecoBase/Shower.h"

#include <iomanip>
#include <utility> // std::move()

namespace recob {

  // the value is assigned in the class definition:
  constexpr double Shower::InvalidOpeningAngle;

  //----------------------------------------------------------------------
  Shower::Shower() {}

  //----------------------------------------------------------------------
  Shower::Shower(TVector3 const& dcosVtx,
                 TVector3 const& dcosVtxErr,
                 TVector3 const& xyz,
                 TVector3 const& xyzErr,
                 std::vector<double> TotalEnergy,
                 std::vector<double> TotalEnergyErr,
                 std::vector<double> dEdx,
                 std::vector<double> dEdxErr,
                 int bestplane,
                 int id,
                 double length,
                 double openAngle)
    : fID(id)
    , fDCosStart(dcosVtx)
    , fSigmaDCosStart(dcosVtxErr)
    , fXYZstart(xyz)
    , fSigmaXYZstart(xyzErr)
    , fTotalEnergy(std::move(TotalEnergy))
    , fSigmaTotalEnergy(std::move(TotalEnergyErr))
    , fdEdx(std::move(dEdx))
    , fSigmadEdx(std::move(dEdxErr))
    , fBestPlane(bestplane)
    , fLength(length)
    , fOpenAngle(openAngle)
  {}

  //----------------------------------------------------------------------
  std::ostream& operator<<(std::ostream& o, Shower const& a)
  {
    o << std::setiosflags(std::ios::fixed) << std::setprecision(3);
    o << " Shower ID " << std::setw(4) << std::right << a.ID();
    o << " Energy    " << std::setw(4) << std::right << a.Energy()[a.best_plane()];
    o << " dEdx    " << std::setw(4) << std::right << a.dEdx()[a.best_plane()];
    return o;
  }

  //----------------------------------------------------------------------
  // < operator.
  //
  bool operator<(const Shower& a, const Shower& b)
  {
    if (a.ID() != b.ID()) return a.ID() < b.ID();

    return false; //They are equal
  }

}
